@@ -6,6 +6,7 @@ extern crate proc_macro; // Apparently needed to be imported like this.
 
 mod constants;
 mod functions;
+mod service;
 mod types;
 mod util;
 
@@ -71,7 +72,7 @@ use syn::{parse_macro_input, AttributeArgs};
 ///
 /// | Pattern | On |  Explanation |
 /// | --- | --- | ---  |
-/// | `success_enum` | `enum` | Denotes this as a [`SuccessEnum`](interoptopus::patterns::successenum::SuccessEnum). |
+/// | `ffi_error` | `enum` | Denotes this as a [`FFIError`](interoptopus::patterns::result::FFIError). |
 ///
 /// # Examples
 ///
@@ -182,3 +183,28 @@ pub fn ffi_constant(attr: TokenStream, item: TokenStream) -> TokenStream {
 
     rval.into()
 }
+
+/// Make a `impl Service {}` block a FFI service.<sup>🚧</sup>
+///
+/// TODO
+#[proc_macro_attribute] // Can now be used as `#[my_attribute]`
+pub fn ffi_service(attr: TokenStream, item: TokenStream) -> TokenStream {
+    let input = proc_macro2::TokenStream::from(item);
+    let attr_args = parse_macro_input!(attr as AttributeArgs);
+
+    let rval = service::ffi_service(attr_args, input);
+
+    rval.into()
+}
+
+/// Inside a `#[ffi_service]` block, mark the FFI constructor.<sup>🚧</sup>
+#[proc_macro_attribute]
+pub fn ffi_service_ctor(_attr: TokenStream, item: TokenStream) -> TokenStream {
+    item
+}
+
+/// Inside a `#[ffi_service]` block, provide special directives to functions.<sup>🚧</sup>
+#[proc_macro_attribute]
+pub fn ffi_service_method(_attr: TokenStream, item: TokenStream) -> TokenStream {
+    item
+}
